/*----------------------------------------------------------------------------
* Testprogramm fuer ubasic_avr-Interpreter....
* 
* Uwe Berger (bergeruw@gmx.net); 2010
* 
* 
* Ein-/Ausgaben erfolgen ueber die serielle Schnittstelle (9600, 8, n, 1)
*
* Kommandos:
*        list -->   gibt das aktuelle Basic-Programm aus
*        run  -->   startet das aktuell geladene Basic-Programm; das
*                   laufende Programm kann man mit einem [ESC] abbrechen
*        load -->   die folgenden Eingaben werden als Basic-Programm gewertet
*                   mit einem [ESC] beendet man den Eingabe-Mode
*        load n --> laed das Programm mit der angegenen Nummer n (siehe
*                   auch ubasic_tests.h)
*
* Have fun!
* ---------
* 
* ----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "usart.h"
#include "ubasic.h"
#include "ubasic_tests.h"

//Taktfrequenz
#ifndef F_CPU
#define F_CPU 16000000UL
#endif

//Baudrate der seriellen Schnittstelle
#define BAUDRATE 57600



#define MAX_PROGRAM_LEN	350

static char program[MAX_PROGRAM_LEN] = "";

//----------------------------------------------------------------------------
void basic_load( char* Buffer){
	int i = 0;
	char c;

	while((c=usart_receive_char()) != 0x1b && i < MAX_PROGRAM_LEN - 1) {
		if (c=='\r' || c=='\n') {
			c='\n';
			usart_write("\r\n");
		} else {
			usart_write_char(c);
		}
		Buffer[i++]=c;
	}
	Buffer[i] = '\0';
	usart_write_str("\r\nEnde Eingabemodus...\r\n");
}

//----------------------------------------------------------------------------
void basic_list(char *c) {
	while (*c) {
		if (*c == '\n') usart_write_char('\r');
		usart_write_char(*c++);
	}
}

//**********************************************************************
char rf(const char *p) {
	return pgm_read_byte(p);
}

//----------------------------------------------------------------------------
void load_from_flash(char* Buffer, int prog_idx) {
	const char *pstrflash;
	if (prog_idx >= (sizeof(progs)/sizeof(progs[0]))) {
		usart_write_str("\r\nProgramm existiert nicht!\r\n");
	} else {
		pstrflash = (const char*)(pgm_read_word(&(progs[prog_idx])));
		strncpy_P(Buffer, pstrflash, MAX_PROGRAM_LEN);
		usart_write("\n\rProgramm %i geladen.\n\r", prog_idx);
	}
}

//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  
	char buffer[24];
	
    usart_init(BAUDRATE); // setup the UART
	
	usart_write("AVR-uBasic-Interpreter; Uwe Berger, 2010\r\n");
	usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
	usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	
	
	while(1){

		char* command = buffer;
        if(usart_read_line(command, sizeof(buffer)) < 1) continue;
        // Basic-Programm starten
        if (strcmp_P(command, PSTR("run")) == 0) {
			usart_write_str("\r\n");
			ubasic_init(program);
			do {
				ubasic_run();
				// Basic-Programm unterbrechen, wenn [ESC] empfangen
				if (usart_is_receive() && (usart_receive_char() == 0x1b)) {
					usart_write_str("\r\nProgramm unterbrochen...!\r\n");
					break;
				}
			} while(!ubasic_finished());
        } else
        // Programm aus Programmflash laden
        if (strncmp_P(command, PSTR("load "), 5) == 0) {
			command += 5;
			if(command[0] == '\0') continue;
			load_from_flash(program, atoi(command));
        } else
        // Programmeingabe via serieller Schnittstelle
        if (strcmp_P(command, PSTR("load")) == 0) {
			usart_write_str("\r\nBeginn manueller Eingabemodus, Ende mit [ESC].\r\n");
			basic_load(program);			
        } else
        // Basic-Programm auflisten
        if (strcmp_P(command, PSTR("list")) == 0) {
			usart_write_str("\r\n");
			basic_list(program);
        } else 	usart_write("\r\nKommando %s unbekannt!\r\n", command);
    }
		
return(0);
}

